@@ -17,29 +17,25 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
-
 #include <config.h>
+
 #include <unistd.h>
 #include <string.h>
-#include <glib/gi18n-lib.h>
-#include <glib/gstdio.h>
-#include <gio/gio.h>
 #include <sys/wait.h>
 #include <stdlib.h>
 #include <errno.h>
 
+#include <glib.h>
+#include <glib/gi18n-lib.h>
+#include <glib/gstdio.h>
+#include <gio/gio.h>
+
 #include "comics-document.h"
 #include "ev-document-misc.h"
 #include "ev-document-thumbnails.h"
 #include "ev-file-helpers.h"
 
-struct _ComicsDocumentClass
-{
-	GObjectClass parent_class;
-};
- 
-typedef enum 
+typedef enum
 {
 	RARLABS,
 	GNAUNRAR,
@@ -47,12 +43,19 @@ typedef enum
 	P7ZIP
 } ComicBookDecompressType;
 
+typedef struct _ComicsDocumentClass ComicsDocumentClass;
+
+struct _ComicsDocumentClass
+{
+	EvDocumentClass parent_class;
+};
+
 struct _ComicsDocument
 {
-	GObject parent_instance;
+	EvDocument parent_instance;
+
 	gchar    *archive, *dir;
-	GSList   *page_names;
-	gint     n_pages;
+	GPtrArray *page_names;
 	gchar    *selected_command;
 	gchar    *extract_command, *list_command, *decompress_tmp;
 	gboolean regex_arg;
@@ -66,21 +69,40 @@ struct _ComicsDocument
 /* For perfomance reasons of 7z* we've choosen to decompress on the temporary 
  * directory instead of decompressing on the stdout */
 
-struct {
-	char *extract, *list, *decompress_tmp; 
-	gboolean regex_arg;
-	gint offset;
-} command_usage_def[] = {
+/**
+ * @extract: command line arguments to pass to extract a file from the archive
+ *   to stdout. The archive file and the file to extract will be appended after
+ *   a "--".
+ * @list: command line arguments to list the archive contents
+ * @decompress_tmp: command line arguments to pass to extract the archive
+ *   into a directory. The archive file and the directory to extract to will be
+ *   appended after a "--".
+ * @regex_arg: whether the command expects one filename or accepts a regex (glob?)
+ * @offset: the byte offset of the filename on each line in the output of
+ *   running the @list command
+ */
+typedef struct {
+        char *extract;
+        char *list;
+        char *decompress_tmp;
+        gboolean regex_arg;
+        gint offset;
+} ComicBookDecompressCommand;
+
+static const ComicBookDecompressCommand command_usage_def[] = {
+        /* RARLABS unrar */
 	{"%s p -c- -ierr", "%s vb -c- -- %s", NULL	       , FALSE, NO_OFFSET},
+
+        /* GNA! unrar */
 	{NULL		 , "%s t %s"	    , "%s -xf %s %s"   , TRUE , NO_OFFSET},
+
+        /* unzip */
 	{"%s -p -C"	 , "%s -Z -1 -- %s" , NULL	       , TRUE , NO_OFFSET},
+
+        /* 7zip */
 	{NULL		 , "%s l -- %s"	    , "%s x -y %s -o%s", FALSE, OFFSET_7Z}
 };
 
-
-typedef struct    _ComicsDocumentClass ComicsDocumentClass;
-
-static void       comics_document_document_iface_init (EvDocumentIface *iface);
 static void       comics_document_document_thumbnails_iface_init (EvDocumentThumbnailsIface *iface);
 
 static GSList*    get_supported_image_extensions (void);
@@ -206,32 +228,20 @@ comics_generate_command_lines (ComicsDocument *comics_document,
 	comics_document->regex_arg = command_usage_def[type].regex_arg;
 	comics_document->offset = command_usage_def[type].offset;
 	if (command_usage_def[type].decompress_tmp) {
-		comics_document->dir = ev_tmp_directory (NULL); 
-		comics_document->decompress_tmp = 
+		comics_document->dir = ev_mkdtemp ("evince-comics-XXXXXX", error);
+                if (comics_document->dir == NULL)
+                        return FALSE;
+
+		/* unrar-free can't create directories, but ev_mkdtemp already created the dir */
+
+		comics_document->decompress_tmp =
 			g_strdup_printf (command_usage_def[type].decompress_tmp, 
 					 comics_document->selected_command, 
 					 quoted_file, 
 					 comics_document->dir);
 		g_free (quoted_file);
-		/* unrar-free can't create directories so we do it on its 
-		 * behalf */
-		if (type == GNAUNRAR) {
-			if (g_mkdir_with_parents (comics_document->dir, 0700) != 
-			    0) {
-				int errsv = errno;
-				g_set_error (error,
-					     EV_DOCUMENT_ERROR,
-					     EV_DOCUMENT_ERROR_INVALID,
-					     _("Failed to create a temporary "
-					     "directory."));
-				g_warning ("Failed to create directory %s: %s", 
-					   comics_document->dir, 
-					   g_strerror (errsv));
-				
-				return FALSE;
-			}
-		}
-		if (!comics_decompress_temp_dir (comics_document->decompress_tmp, 
+
+		if (!comics_decompress_temp_dir (comics_document->decompress_tmp,
 		    comics_document->selected_command, error))
 			return FALSE;
 		else
@@ -356,6 +366,13 @@ comics_check_decompress_command	(gchar          *mime_type,
 	return FALSE;
 }
 
+static int
+sort_page_names (gconstpointer a,
+                 gconstpointer b)
+{
+  return strcmp (* (const char **) a, * (const char **) b);
+}
+
 static gboolean
 comics_document_load (EvDocument *document,
 		      const char *uri,
@@ -425,6 +442,8 @@ comics_document_load (EvDocument *document,
 		return FALSE;
 	}
 
+        comics_document->page_names = g_ptr_array_sized_new (64);
+
 	supported_extensions = get_supported_image_extensions ();
 	for (i = 0; cb_files[i] != NULL; i++) {
 		if (comics_document->offset != NO_OFFSET) {
@@ -445,12 +464,8 @@ comics_document_load (EvDocument *document,
 		suffix = g_ascii_strdown (suffix + 1, -1);
 		if (g_slist_find_custom (supported_extensions, suffix,
 					 (GCompareFunc) strcmp) != NULL) {
-			comics_document->page_names =
-				g_slist_insert_sorted (
-					comics_document->page_names,
-					g_strdup (g_strstrip (cb_file)),
-					(GCompareFunc) strcmp);
-			comics_document->n_pages++;
+                        g_ptr_array_add (comics_document->page_names,
+                                         g_strstrip (g_strdup (cb_file)));
 		}
 		g_free (suffix);
 	}
@@ -458,7 +473,7 @@ comics_document_load (EvDocument *document,
 	g_slist_foreach (supported_extensions, (GFunc) g_free, NULL);
 	g_slist_free (supported_extensions);
 
-	if (comics_document->n_pages == 0) {
+	if (comics_document->page_names->len == 0) {
 		g_set_error (error,
 			     EV_DOCUMENT_ERROR,
 			     EV_DOCUMENT_ERROR_INVALID,
@@ -466,6 +481,10 @@ comics_document_load (EvDocument *document,
 			     uri);
 		return FALSE;
 	}
+
+        /* Now sort the pages */
+        g_ptr_array_sort (comics_document->page_names, sort_page_names);
+
 	return TRUE;
 }
 
@@ -483,7 +502,12 @@ comics_document_save (EvDocument *document,
 static int
 comics_document_get_n_pages (EvDocument *document)
 {
-	return COMICS_DOCUMENT (document)->n_pages;
+	ComicsDocument *comics_document = COMICS_DOCUMENT (document);
+
+        if (comics_document->page_names == NULL)
+                return 0;
+
+	return comics_document->page_names->len;
 }
 
 static void
@@ -542,10 +566,8 @@ comics_document_get_page_size (EvDocument *document,
 		g_spawn_close_pid (child_pid);
 		g_object_unref (loader);
 	} else {
-		filename = g_build_filename (comics_document->dir, 	
-					     (char*) g_slist_nth_data (
-					     comics_document->page_names, 
-					     page->index),
+		filename = g_build_filename (comics_document->dir,
+                                             (char *) comics_document->page_names->pdata[page->index],
 					     NULL);
 		pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
 		g_free (filename);
@@ -617,9 +639,7 @@ comics_document_render_pixbuf (EvDocument      *document,
 	} else {
 		filename = 
 			g_build_filename (comics_document->dir,
-					  (char*) g_slist_nth_data (
-						comics_document->page_names, 
-						rc->page->index),
+                                          (char *) comics_document->page_names->pdata[rc->page->index],
 					  NULL);
 	   
 		gdk_pixbuf_get_file_info (filename, &width, &height);
@@ -704,13 +724,11 @@ comics_document_finalize (GObject *object)
 			g_warning (_("There was an error deleting “%s”."),
 				   comics_document->dir);
 		g_free (comics_document->dir);
-		g_remove (ev_tmp_dir ());
 	}
 	
 	if (comics_document->page_names) {
-		g_slist_foreach (comics_document->page_names,
-				 (GFunc) g_free, NULL);
-		g_slist_free (comics_document->page_names);
+                g_ptr_array_foreach (comics_document->page_names, (GFunc) g_free, NULL);
+                g_ptr_array_free (comics_document->page_names, TRUE);
 	}
 
 	g_free (comics_document->archive);
@@ -724,27 +742,16 @@ comics_document_finalize (GObject *object)
 static void
 comics_document_class_init (ComicsDocumentClass *klass)
 {
-	GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
+	GObjectClass    *gobject_class = G_OBJECT_CLASS (klass);
+	EvDocumentClass *ev_document_class = EV_DOCUMENT_CLASS (klass);
+
 	gobject_class->finalize = comics_document_finalize;
-}
 
-static EvDocumentInfo *
-comics_document_get_info (EvDocument *document)
-{
-	EvDocumentInfo *info;
-	info = g_new0 (EvDocumentInfo, 1);
-	return info;
-}
-
-static void
-comics_document_document_iface_init (EvDocumentIface *iface)
-{
-	iface->load = comics_document_load;
-	iface->save = comics_document_save;
-	iface->get_n_pages = comics_document_get_n_pages;
-	iface->get_page_size = comics_document_get_page_size;
-	iface->render = comics_document_render;
-	iface->get_info = comics_document_get_info;
+	ev_document_class->load = comics_document_load;
+	ev_document_class->save = comics_document_save;
+	ev_document_class->get_n_pages = comics_document_get_n_pages;
+	ev_document_class->get_page_size = comics_document_get_page_size;
+	ev_document_class->render = comics_document_render;
 }
 
 static void
@@ -753,7 +760,6 @@ comics_document_init (ComicsDocument *comics_document)
 	comics_document->archive = NULL;
 	comics_document->page_names = NULL;
 	comics_document->extract_command = NULL;
-	comics_document->n_pages = 0;
 }
 
 /* Returns a list of file extensions supported by gdk-pixbuf */
@@ -834,13 +840,14 @@ extract_argv (EvDocument *document, gint page)
 	char *command_line, *quoted_archive, *quoted_filename;
 	GError *err = NULL;
 
+        if (page >= comics_document->page_names->len)
+                return NULL;
+
 	quoted_archive = g_shell_quote (comics_document->archive);
 	if (comics_document->regex_arg) {
-		quoted_filename = comics_regex_quote (
-			g_slist_nth_data (comics_document->page_names, page));
+		quoted_filename = comics_regex_quote (comics_document->page_names->pdata[page]);
 	} else {
-		quoted_filename = g_shell_quote (
-			g_slist_nth_data (comics_document->page_names, page));
+		quoted_filename = g_shell_quote (comics_document->page_names->pdata[page]);
 	}
 
 	command_line = g_strdup_printf ("%s -- %s %s",
@@ -849,13 +856,13 @@ extract_argv (EvDocument *document, gint page)
 					quoted_filename);
 
 	g_shell_parse_argv (command_line, NULL, &argv, &err);
-	
+
 	if (err) {
 		g_warning (_("Error %s"), err->message);
 		g_error_free (err);
 		return NULL;
 	}
-	
+
 	g_free (command_line);
 	g_free (quoted_archive);
 	g_free (quoted_filename);
